import { DSNComponents, DSNLike, DSNProtocol } from '@sentry/types';
/** The Sentry DSN, identifying a Sentry instance and project. */
export declare class DSN implements DSNComponents {
    /** Protocol used to connect to Sentry. */
    protocol: DSNProtocol;
    /** Public authorization key. */
    user: string;
    /** Private authorization key (deprecated, optional). */
    pass: string;
    /** Hostname of the Sentry instance. */
    host: string;
    /** Port of the Sentry instance. */
    port: string;
    /** Path */
    path: string;
    /** Project ID */
    projectId: string;
    /** Creates a new DSN component */
    constructor(from: DSNLike);
    /**
     * Renders the string representation of this DSN.
     *
     * By default, this will render the public representation without the password
     * component. To get the deprecated private representation, set `withPassword`
     * to true.
     *
     * @param withPassword When set to true, the password will be included.
     */
    toString(withPassword?: boolean): string;
    /** Parses a string into this DSN. */
    private fromString;
    /** Maps DSN components into this instance. */
    private fromComponents;
    /** Validates this DSN and throws on error. */
    private validate;
}
